//
// Created by philipp on 04.04.18.
//

#include "../include/PlatoonProtocolLib/VehicleFacade.h"
#include "../include/PlatoonProtocolLib/LeaderVehicle.h"
#include "../include/PlatoonProtocolLib/FollowerVehicle.h"

namespace platoonProtocol
{

VehicleFacade::VehicleFacade(Vehicle::Role role, networking::Networking & net, const NetworkInfo & info)
{
    reset(role, net, info);
}

void VehicleFacade::reset(Vehicle::Role role, networking::Networking & net, const NetworkInfo & info)
{
    std::lock_guard<std::mutex> lock(mutex);
    if (role == Vehicle::Role::LEADER)
        vehicle = LeaderVehicle::create(net, info);
    else
        vehicle = FollowerVehicle::create(net, info);
}

Vehicle::Ptr VehicleFacade::get()
{
    std::lock_guard<std::mutex> lock(mutex);
    return vehicle;
}

bool VehicleFacade::isIdle()
{
    std::lock_guard<std::mutex> lock(mutex);
    return vehicle->isIdle();
}

bool VehicleFacade::isPlatoonRunning()
{
    std::lock_guard<std::mutex> lock(mutex);
    return vehicle->isIdle();
}

void VehicleFacade::createPlatoon()
{
    std::lock_guard<std::mutex> lock(mutex);
    vehicle->createPlatoon();
}

void VehicleFacade::leavePlatoon()
{
    std::lock_guard<std::mutex> lock(mutex);
    vehicle->leavePlatoon();
}

void VehicleFacade::stop()
{
    std::lock_guard<std::mutex> lock(mutex);
    vehicle->stop();
}

VehicleId VehicleFacade::getOwnVehicleId()
{
    std::lock_guard<std::mutex> lock(mutex);
    return vehicle->getOwnVehicleId();
}

Vehicle::Role VehicleFacade::getRole()
{
    std::lock_guard<std::mutex> lock(mutex);
    return vehicle->getRole();
}

void VehicleFacade::setOnRunningPlatoonCallback(const Vehicle::Callback & callback)
{
    std::lock_guard<std::mutex> lock(mutex);
    vehicle->setOnRunningPlatoonCallback(callback);
}

void VehicleFacade::setOnLeavingPlatoonCallback(const Vehicle::Callback & callback)
{
    std::lock_guard<std::mutex> lock(mutex);
    vehicle->setOnLeavingPlatoonCallback(callback);
}

std::size_t VehicleFacade::getNumFollowers()
{
    std::lock_guard<std::mutex> lock(mutex);
    return std::dynamic_pointer_cast<LeaderVehicle>(vehicle)->getNumFollowers();
}

void VehicleFacade::setPlatoonConfig(const PlatoonConfig & platoonConfig)
{
    std::lock_guard<std::mutex> lock(mutex);
    return std::dynamic_pointer_cast<LeaderVehicle>(vehicle)->setPlatoonConfig(platoonConfig);
}

PlatoonConfig VehicleFacade::getPlatoonConfig()
{
    std::lock_guard<std::mutex> lock(mutex);
    return std::dynamic_pointer_cast<FollowerVehicle>(vehicle)->getPlatoonConfig();
}

void VehicleFacade::setOnPlatoonConfigUpdatedCallback(const Vehicle::Callback & callback)
{
    std::lock_guard<std::mutex> lock(mutex);
    return std::dynamic_pointer_cast<FollowerVehicle>(vehicle)->setOnPlatoonConfigUpdatedCallback(callback);
}


}